% neurips_2026.cls — NeurIPS 2026 document class (simplified)
% Based on the standard NeurIPS template format

\NeedsTeXFormat{LaTeX2e}
\ProvidesClass{neurips_2026}[2026/01/01 NeurIPS 2026 Style]

% Options
\newif\if@final\@finalfalse
\DeclareOption{final}{\@finaltrue}
\DeclareOption{preprint}{}
\DeclareOption*{\PassOptionsToClass{\CurrentOption}{article}}
\ProcessOptions\relax

% Base class
\LoadClass[11pt]{article}

% Required packages
\RequirePackage[verbose=true,letterpaper]{geometry}
\AtBeginDocument{
  \newgeometry{
    textheight=9in,
    textwidth=5.5in,
    top=1in,
    headheight=12pt,
    headsep=25pt,
    footskip=30pt
  }
}

\RequirePackage{natbib}
\RequirePackage{fancyhdr}

% Fonts
\renewcommand{\rmdefault}{ptm}
\renewcommand{\sfdefault}{phv}

% No page numbers in submission
\if@final
  \pagestyle{fancy}
  \fancyhf{}
  \fancyfoot[C]{\thepage}
  \renewcommand{\headrulewidth}{0pt}
\else
  \pagestyle{empty}
\fi

% Title
\def\@maketitle{%
  \vbox to 0pt{}%
  \vskip -\headsep
  \vskip 0.3in
  \begin{center}%
    {\LARGE\bf \@title \par}%
    \vskip 0.3in plus 1fil minus 0.1in
    {
      \def\and{%
        \end{tabular}\hfil\linebreak[0]\hfil%
        \begin{tabular}[t]{c}\bf\rule{\z@}{24pt}\ignorespaces}%
      \begin{tabular}[t]{c}\bf\rule{\z@}{24pt}\@author\end{tabular}%
    }
    \vskip 0.3in plus 2fil minus 0.1in
  \end{center}
}

% Section formatting
\renewcommand{\section}{\@startsection{section}{1}{\z@}%
  {-2.0ex \@plus -0.5ex \@minus -0.2ex}%
  {1.5ex \@plus 0.3ex \@minus 0.2ex}%
  {\large\bf\raggedright}}
\renewcommand{\subsection}{\@startsection{subsection}{2}{\z@}%
  {-1.8ex \@plus -0.5ex \@minus -0.2ex}%
  {0.8ex \@plus 0.2ex}%
  {\normalsize\bf\raggedright}}
\renewcommand{\subsubsection}{\@startsection{subsubsection}{3}{\z@}%
  {-1.5ex \@plus -0.5ex \@minus -0.2ex}%
  {0.5ex \@plus 0.2ex}%
  {\normalsize\bf\raggedright}}
\renewcommand{\paragraph}{\@startsection{paragraph}{4}{\z@}%
  {1.0ex \@plus 0.5ex \@minus 0.2ex}%
  {-1em}%
  {\normalsize\bf}}

% Abstract
\renewenvironment{abstract}%
{\vskip 0.075in\centerline{\large\bf Abstract}\vspace{0.5ex}\begin{quote}}%
{\par\end{quote}\vskip 1ex}

\endinput
